// Allowed sites extension
// August 2004, Jesse Ruderman (IBM)

try{


const DEBUG_NAME = "Allowed sites JS Component";

function myDump (msg) 
{
  var fullmsg = DEBUG_NAME + ": " + msg;
  
  dump(fullmsg + "\n");
  
  try {
    var cns = Components.classes["@mozilla.org/consoleservice;1"]
                          .getService(Components.interfaces.nsIConsoleService);
    cns.logStringMessage(fullmsg);
  }
  catch (e) {
    dump("Can't put that in the JS console.\n");
  }
}


/******************************************************************************/


// What's the difference between REJECT_REQUEST and REJECT_OTHER?
const ACCEPT = Components.interfaces.nsIContentPolicy.ACCEPT;
const REJECT = Components.interfaces.nsIContentPolicy.REJECT_REQUEST;

/*
 * // For 0.9.3
 * const ACCEPT = true;
 * const REJECT = true;
 */


var rules = [];

var inited = false;

function init ()
{
  var jsloader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
  try {
    try {
    jsloader.loadSubScript('chrome://jslib/content/jslib.js');
    }
    catch(er) { throw("Failed to load jslib. Did you install jslib?"); } 

	  include('chrome://jslib/content/io/dirUtils.js');
	  include('chrome://jslib/content/io/dir.js');
	  include('chrome://jslib/content/io/file.js');
	  
	  var profileDirectory = (new DirUtils()).getMozUserHomeDir();
	  var fn = new Dir(profileDirectory)
	  fn.append("allowed-sites.txt");
	  
	  if (!fn.exists())
	  {
	    myDump("Allowed-sites.txt doesn't exist in profile directory, using default.");
	    
	    fn2 = new Dir(profileDirectory);
	    fn2.append("extensions");
	    fn2.append("{B33DD728-C988-4fc8-92B4-B09829541EF2}"); // from install.rdf
	    fn2.append("defaults");
	    fn2.append("allowed-sites.txt");
	    
	    var file2 = new File(fn2.path); // this can fail if installed globally instead of in profile!
	    file2.copy(fn.path);
	  }
	  
	  var file = new File(fn.path);
	  file.open();
	  
	  while (!file.EOF) {
	    var line = file.readline();
	    addRule(line);
	  }
	}
	catch(er) { myDump("Error while initing: " + er); }
  
  //myDump(rules);
  
  inited = true;
}



function prefixChecker(prefix)
{
  var len = prefix.length;
  
  return function(s) 
  {
    return (s.substr(0,len) == prefix);
  } 
}

function exactChecker(url)
{
  return function(s)
  {
    return (s == url);
  }
}

function addRule(line)
{
  if (line.length == 0)
    return;
  if (line[0] == '#')
    return;

  // This would be wrong if things could have a "last parameter" containing spaces (like in the IRC protocol).
  words = line.split(" ");

  var allowed;

  if (words[0] == "ALLOW")
    allowed = ACCEPT;
  else if (words[0] == "DISALLOW")
    allowed = REJECT;
  else 
    throw("Allowed value should be ALLOW or DISALLOW, not: " + words[0]);

  var matcher;

  if (words[1] == "regexp")
    matcher = new RegExp(words[2]);
  else if (words[1] == "prefix")
    matcher = prefixChecker(words[2]);
  else if (words[1] == "exact")
    matcher = exactChecker(words[2]);
  else if (words[1] == "all")
    matcher = function () { return true; }
  else
    throw("Unknown match type: " + words[1]);

  rules.push ( [allowed, matcher, line] );
}




/******************************************************************************/

function MyComponent() {}

MyComponent.prototype = {

    QueryInterface: function (iid) {
      if (!(
            iid.equals(Components.interfaces.nsIContentPolicy)
		     || iid.equals(Components.interfaces.nsISupports)
		      ))
          throw Components.results.NS_ERROR_NO_INTERFACE;
      return this;
    },

  /**
   * nsIContentPolicy::shouldLoad
   *
   * Should the resource at this location be loaded?
   * ShouldLoad will be called before loading the resource at aContentLocation
   * to determine whether to start the load at all.
   *
   * @param aContentType      the type of content being tested. This will be one
   *                          one of the TYPE_* constants.
   *
   * @param aContentLocation  the location of the content being checked; must
   *                          not be null
   *
   * @param aRequestOrigin    OPTIONAL. the location of the resource that
   *                          initiated this load request; can be null if
   *                          inapplicable
   *
   * @param aRequestingNode   OPTIONAL. the DOM node that initiated the
   *                          request; can be null if inapplicable
   *
   * @param aMimeTypeGuess    OPTIONAL. a guess for the requested content's
   *                          MIME type, based on information available to
   *                          the request initiator (e.g., an OBJECT's type
   *                          attribute); does not reliably reflect the
   *                          actual MIME type of the requested content
   *
   * @param aExtra            an OPTIONAL argument, pass-through for non-Gecko
   *                          callers to pass extra data to callees.
   *
   * @return ACCEPT or REJECT_*
   */
    shouldLoad : function ( aContentType,     // unsigned long
                            aContentLocation, // nsIURI
                            aRequestOrigin,   // nsIURI
                            aRequestingNode,  // nsIDOMNode
                            aMimeTypeGuess,   // ACString
                            aExtra)           // nsISupports
    {
			if (!inited) {
			  myDump("Trying to init...");
				try { init(); } catch(er) { myDump(er); }
			}
			
			try {
			 	var URL = aContentLocation.spec;
			
				for (i = 0; rule = rules[i]; ++i) {
				  if (rule[1](URL)) {
				    if (rule[0] == REJECT)
				      myDump("Disallowing load of " + URL + " because the first line in your allowed-sites.txt the URL matches is: " + rule[2]);
				    return rule[0];
				  }
			  }
			
		 	  myDump("No rule matched.  That means there wasn't an 'all' rule.");
	 		  return ACCEPT;
			}
			catch(er) { myDump(er); return ACCEPT; }
    },

  /**
   * nsIContentPolicy::shouldProcess
   *
   * Should the resource be processed?
   * ShouldProcess will be called once all the information passed to it has
   * been determined about the resource, typically after part of the resource
   * has been loaded.
   *
   * @param aContentType      the type of content being tested. This will be one
   *                          one of the TYPE_* constants.
   *
   * @param aContentLocation  OPTIONAL; the location of the resource being
   *                          requested: MAY be, e.g., a post-redirection URI
   *                          for the resource.
   *
   * @param aRequestOrigin    OPTIONAL. the location of the resource that
   *                          initiated this load request; can be null if
   *                          inapplicable
   *
   * @param aRequestingNode   OPTIONAL. the DOM node that initiated the
   *                          request; can be null if inapplicable
   *
   * @param aMimeType         the MIME type of the requested resource (e.g.,
   *                          image/png), as reported by the networking library,
   *                          if available (may be empty if inappropriate for
   *                          the type, e.g., TYPE_REFRESH).
   *
   * @param aExtra            an OPTIONAL argument, pass-through for non-Gecko
   *                          callers to pass extra data to callees.
   *
   * @return ACCEPT or REJECT_*
   */
    shouldProcess : function ( aContentType,     // unsigned long
                               aContentLocation, // nsIURI
                               aRequestOrigin,   // nsIURI
                               aRequestingNode,  // nsIDOMNode
                               aMimeTypeGuess,   // ACString - no longer a guess!
                               aExtra)           // nsISupports
    {
      // myDump("shouldProcess");
      
      // This never seems to be called.
      // I don't know when this is supposed to be called.
      // (Instead of shouldLoad, always but after shouldLoad, or
      // only if shouldLoad was given incomplete information.)
      
      return shouldLoad(aContentType, 
                        aContentLocation, 
                        aRequestOrigin, 
                        aRequestingNode,
                        aMimeTypeGuess, 
                        aExtra);
    },
}

/******************************************************************************/

var myModule = {

    myClassID: Components.ID("{EDD6E53E-26DB-411c-A193-1AF843365380}"),
    myContractID: "@squarefree.com/allowed-sites;1",
    myClassName: "Allowed-sites",

    registerSelf: function (compMgr, fileSpec, location, type) {
      myDump(" ***** Registering: " + DEBUG_NAME + " ****");

      compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
      compMgr.registerFactoryLocation(this.myClassID,
                                      this.myClassName,
                                      this.myContractID,
                                      fileSpec,
                                      location,
                                      type);

        /* Register as a content-policy so nsContentPolicyUtils users will call our functions. */
      var catman = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
      catman.addCategoryEntry("content-policy",
                              this.myContractID, // should be class name instead? doesn't matter?
                              this.myContractID,
                              true,
                              true); // sixth param left out?
    },


    // This wasn't in the example Hello World thing, but it was in AdBlock...
  	unregisterSelf: function(compMgr, fileSpec, location) {
      //dump(" ***** Unregistering: " + DEBUG_NAME + " ****\n"); // In shutdown mode, use normal dump.

      compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  		compMgr.unregisterFactoryLocation(this.myClassID, fileSpec);

  		var catman = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
  		catman.deleteCategoryEntry("content-policy", this.myContractID, true);
  	},



    getClassObject: function (compMgr, cid, iid) {
      if (!cid.equals(this.myClassID))
          throw Components.results.NS_ERROR_NO_INTERFACE;

      if (!iid.equals(Components.interfaces.nsIFactory))
          throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

      return this.myFactory;
    },

    myFactory: {

      createInstance: function (outer, iid) {
				myDump("CreateInstance: " + iid);
        
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;

        return (new MyComponent()).QueryInterface(iid);
      }
    },

    canUnload: function(compMgr) {
      //dump("****** About to unload: " + DEBUG_NAME + " *****\n"); // In shutdown mode, use normal dump.
      return true;
    }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }

/******************************************************************************/


}catch(ex){alert(ex)}